package main

import (
	"os"
	"path"
	"text/template"

	"github.com/BurntSushi/toml"
)

// Config contains site configuration.
type Config struct {
	Title     string            `toml:"title"` // site title
	URLs      []string          `toml:"urls"`  // site URLs
	Feeds     map[string]string `toml:"feeds"` // site feeds
	Tasks     map[string]*Task  `toml:"tasks"` // site tasks
	Templates *Templates        `toml:"-"`     // site templates
}

// Task represents a site build task.
type Task struct {
	InputExt    string `toml:"input_ext"`    // input file extension
	OutputExt   string `toml:"output_ext"`   // output file extension
	TemplateExt string `toml:"template_ext"` // template file extension
	PreProcess  string `toml:"preprocess"`   // preprocess command
	PostProcess string `toml:"postprocess"`  // postprocess command
	StaticDir   string `toml:"static_dir"`   // static file directory
	OutputDir   string `toml:"output_dir"`   // output directory
}

func (t Task) OutputPath(pagePath string) string {
	return path.Join(pagePath, "index"+t.OutputExt)
}

// LoadConfig loads the configuration from the provided path.
func LoadConfig(path string) (*Config, error) {
	f, err := os.Open(path)
	if err != nil {
		return nil, err
	}
	defer f.Close()

	c := &Config{}
	if _, err := toml.DecodeReader(f, c); err != nil {
		return nil, err
	}

	return c, nil
}

// LoadTemplates loads templates from the provided path.
func (c *Config) LoadTemplates(path string) error {
	// Site contains site metadata passed to templates
	type Site struct {
		Title string
		URLs  []string
	}

	// Load templates
	c.Templates = NewTemplates()
	c.Templates.Funcs(template.FuncMap{
		"site": func() Site {
			return Site{
				Title: c.Title,
				URLs:  c.URLs,
			}
		},
	})
	return c.Templates.Load(path)
}
